/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

Essais de lectrure (comparaison) entre un ADS1113 et un ADS1115
Conçu pour rouler sur un Arduino-Nano 
Électro-Bidouilleur - Mars 2021

Licence CC BY-SA 4.0: Attribution + Partage dans les Mêmes Conditions 4.0 International.
(https://creativecommons.org/licenses/by-sa/4.0/deed.fr)

*/

#include <Wire.h>   // La librairie pour le I2C

const unsigned char config_high = 0b01110101;    // Les octets de configuration de l'ADS1113/5
const unsigned char config_low  = 0b11100000;    // 000: 8 SPS,  111: 860 SPS
int addr;   // Variable pour l'adresse I2C

// La fonction d'initialisation, façon Arduino IDE
void setup() 
{
  delay(2000);           // Petite pause de stabilisation

  pinMode(6, OUTPUT);    // La broche de LED du nano

  // Configurer le port série
  Serial.begin(115200); 
  Serial.setTimeout(10000);

  // Ouvrir le port I2C
  Wire.begin(); // join i2c bus (address optional for master)

  // Configurer le ADS1113, adresse 72
  addr = 72;
  Wire.beginTransmission(addr);           // Lancer une trasmission vers l'ADC
  Wire.write(1);                          // Mettre le registre pointeur vers l'adresse du registre de configuration (addresse 1)
  Wire.write(config_high); // Bits 15-8   // Écrire les Bits 15-8 de configuration
  Wire.write(config_low); // Bits 7-0     // Écrire les Bits 7-0  de configuration         
  Wire.endTransmission();                 // terminer la transmission

  // Configurer le ADS1115, adresse 73
  addr = 73;
  Wire.beginTransmission(addr);           // Lancer une trasmission vers l'ADC 
  Wire.write(1);                          // Mettre le registre pointeur vers l'adresse du registre de configuration (addresse 1)
  Wire.write(config_high);                // Écrire les Bits 15-8 de configuration
  Wire.write(config_low);                 // Écrire les Bits 7-0  de configuration     
  Wire.endTransmission();                 // terminer la transmission
}

// La fonction qui tourne en boucle, façon Arduino IDE
void loop() 
{
  float adc_volt;           // Variable pour la lecture en Volts
  unsigned int adc_val;     // Variable pour la lecture de l'ADC
  
  // Alterner antre les adresses 72 et 73 dans la boucle
  if (addr == 72) 
  {
    addr = 73; 
    Serial.print("ADS1115: ");
  }
  else 
  {
    delay(1000); 
    addr = 72;
    Serial.print("\nADS1113: ") ;
  }
  //Déclencher une conversion
  Wire.beginTransmission(addr);         // Lancer une trasmission vers l'ADC
  Wire.write(1);                        // Mettre le registre pointeur vers l'adresse du registre de configuration (addresse 1)
  Wire.write(config_high | 10000000);   // Lancer une conversion (bit 15 mis à 1)
  Wire.write(config_low);               // Bits 7-0 inchangés
  Wire.endTransmission();               // terminer la transmission

  // Attendre la fin de la conversion
  digitalWrite(6, HIGH);                // CLignotement de LED
  Wire.requestFrom(addr, 1);
  while (!((Wire.read() & 10000000)>>7))  Wire.requestFrom(addr, 1);    // demander un octet de l'ADC
  digitalWrite(6, LOW);                 // CLignotement de LED

  // Lire la valeur de l'ADC
  Wire.beginTransmission(addr);         // Lancer une trasmission vers l'ADC
  Wire.write(0);                        // envoie l'adresse P1:0 
  Wire.endTransmission();               // stopper la transmission
  Wire.requestFrom(addr, 2);            // demander 2 octets de l'ADC

  // Formatter et calculer la valeur de l'ADC
  adc_val = (Wire.read()<<8 | Wire.read());       // Formatter la lecture des octets de l'ADC en un seul mot de 16 bits
  adc_volt = (float(adc_val) / 32767) * 2.048;    // Calculer l'équivalent en Volts de la lecture de l'ADC.

  // Afficher les résultats
  Serial.print(adc_val,DEC);            // Imprimer les résultats.
  Serial.print("   ");                  //    "
  Serial.print(adc_volt,4); 
  Serial.println(" V"); 
}
